# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from experiment_collection_core import service_pb2 as experiment__collection__core_dot_service__pb2


class ExperimentServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateExperiment = channel.unary_unary(
                '/ExperimentService/CreateExperiment',
                request_serializer=experiment__collection__core_dot_service__pb2.AddExperiment.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.ReserveExperiment = channel.unary_unary(
                '/ExperimentService/ReserveExperiment',
                request_serializer=experiment__collection__core_dot_service__pb2.ReserveExperimentRequest.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.DeleteExperiment = channel.unary_unary(
                '/ExperimentService/DeleteExperiment',
                request_serializer=experiment__collection__core_dot_service__pb2.SimpleExperiment.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.CheckExperiment = channel.unary_unary(
                '/ExperimentService/CheckExperiment',
                request_serializer=experiment__collection__core_dot_service__pb2.SimpleExperiment.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.GetExperiments = channel.unary_unary(
                '/ExperimentService/GetExperiments',
                request_serializer=experiment__collection__core_dot_service__pb2.SimpleNamespace.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.ExperimentsReply.FromString,
                )
        self.CreateNamespace = channel.unary_unary(
                '/ExperimentService/CreateNamespace',
                request_serializer=experiment__collection__core_dot_service__pb2.SimpleNamespace.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.RevokeToken = channel.unary_unary(
                '/ExperimentService/RevokeToken',
                request_serializer=experiment__collection__core_dot_service__pb2.SimpleToken.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )
        self.GrantAccess = channel.unary_unary(
                '/ExperimentService/GrantAccess',
                request_serializer=experiment__collection__core_dot_service__pb2.GrantAccessRequest.SerializeToString,
                response_deserializer=experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
                )


class ExperimentServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateExperiment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReserveExperiment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteExperiment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckExperiment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetExperiments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNamespace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RevokeToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GrantAccess(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExperimentServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateExperiment': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateExperiment,
                    request_deserializer=experiment__collection__core_dot_service__pb2.AddExperiment.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'ReserveExperiment': grpc.unary_unary_rpc_method_handler(
                    servicer.ReserveExperiment,
                    request_deserializer=experiment__collection__core_dot_service__pb2.ReserveExperimentRequest.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'DeleteExperiment': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteExperiment,
                    request_deserializer=experiment__collection__core_dot_service__pb2.SimpleExperiment.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'CheckExperiment': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckExperiment,
                    request_deserializer=experiment__collection__core_dot_service__pb2.SimpleExperiment.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'GetExperiments': grpc.unary_unary_rpc_method_handler(
                    servicer.GetExperiments,
                    request_deserializer=experiment__collection__core_dot_service__pb2.SimpleNamespace.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.ExperimentsReply.SerializeToString,
            ),
            'CreateNamespace': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNamespace,
                    request_deserializer=experiment__collection__core_dot_service__pb2.SimpleNamespace.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'RevokeToken': grpc.unary_unary_rpc_method_handler(
                    servicer.RevokeToken,
                    request_deserializer=experiment__collection__core_dot_service__pb2.SimpleToken.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
            'GrantAccess': grpc.unary_unary_rpc_method_handler(
                    servicer.GrantAccess,
                    request_deserializer=experiment__collection__core_dot_service__pb2.GrantAccessRequest.FromString,
                    response_serializer=experiment__collection__core_dot_service__pb2.SimpleReply.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'ExperimentService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ExperimentService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateExperiment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/CreateExperiment',
            experiment__collection__core_dot_service__pb2.AddExperiment.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReserveExperiment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/ReserveExperiment',
            experiment__collection__core_dot_service__pb2.ReserveExperimentRequest.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteExperiment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/DeleteExperiment',
            experiment__collection__core_dot_service__pb2.SimpleExperiment.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckExperiment(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/CheckExperiment',
            experiment__collection__core_dot_service__pb2.SimpleExperiment.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetExperiments(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/GetExperiments',
            experiment__collection__core_dot_service__pb2.SimpleNamespace.SerializeToString,
            experiment__collection__core_dot_service__pb2.ExperimentsReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateNamespace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/CreateNamespace',
            experiment__collection__core_dot_service__pb2.SimpleNamespace.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RevokeToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/RevokeToken',
            experiment__collection__core_dot_service__pb2.SimpleToken.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GrantAccess(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/ExperimentService/GrantAccess',
            experiment__collection__core_dot_service__pb2.GrantAccessRequest.SerializeToString,
            experiment__collection__core_dot_service__pb2.SimpleReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import datetime
import json
import typing

import grpc
import pandas as pd
from google.protobuf.timestamp_pb2 import Timestamp

from experiment_collection.utils import postprocess_df
from experiment_collection_core import service_pb2_grpc, service_pb2
from .collection_abc import ExperimentCollectionABC
from .experiment import Experiment


class ExperimentCollectionRemoteException(Exception):
    pass


class ExperimentCollectionRemote(ExperimentCollectionABC):
    def __init__(self, host: str, namespace: str, token: str, credentials=None):
        if credentials is None:
            self.channel = grpc.insecure_channel(host)
        else:
            if isinstance(credentials, bool) and credentials:
                credentials = grpc.ssl_channel_credentials()
            self.channel = grpc.secure_channel(host, credentials)
        self.stub = service_pb2_grpc.ExperimentServiceStub(self.channel)
        self.host = host
        self.namespace = namespace
        self.token = token
        self.auth = {'namespace': self.namespace, 'token': self.token}

    def close(self):
        self.channel.close()

    def add_experiment(self, exp: Experiment, *, ignore_included=False):
        ts = Timestamp()
        # pylint: disable=E1101
        ts.FromDatetime(exp.time)
        experiment = service_pb2.Experiment(name=exp.name,
                                            params=json.dumps(exp.params),
                                            metrics=json.dumps(exp.metrics),
                                            time=ts,
                                            )
        r = self.stub.CreateExperiment(service_pb2.AddExperiment(experiment=experiment, **self.auth))
        if not r.status and not ignore_included and self.check_experiment(exp):
            raise ExperimentCollectionRemoteException(r.error)

    def reserve_experiment(self, exp: typing.Union[Experiment, str], duration: int) -> bool:
        assert duration > 0, 'Duration should be more than zero'
        if isinstance(exp, Experiment):
            exp = exp.name
        r = self.stub.ReserveExperiment(
            service_pb2.ReserveExperimentRequest(experiment=exp, duration=duration, **self.auth))
        return r.status

    def delete_experiment(self, exp: typing.Union[Experiment, str]):
        if isinstance(exp, Experiment):
            exp = exp.name
        r = self.stub.DeleteExperiment(service_pb2.SimpleExperiment(experiment=exp, **self.auth))
        if not r.status:
            raise ExperimentCollectionRemoteException(r.error)

    def check_experiment(self, exp: typing.Union[Experiment, str]) -> bool:
        if isinstance(exp, Experiment):
            exp = exp.name
        r = self.stub.CheckExperiment(service_pb2.SimpleExperiment(experiment=exp, **self.auth))
        return r.status

    def get_experiments(self, normalize=True):
        r = self.stub.GetExperiments(service_pb2.SimpleNamespace(**self.auth))
        if not r.status:
            raise ExperimentCollectionRemoteException(r.error)
        experiments = {'name': [], 'params': [], 'metrics': [], 'time': []}
        for exp in r.experiments:
            ts = datetime.datetime.fromtimestamp(exp.time.seconds + exp.time.nanos / 1e9)
            experiments['name'].append(exp.name)
            experiments['params'].append(exp.params)
            experiments['metrics'].append(exp.metrics)
            experiments['time'].append(ts)
        df = pd.DataFrame(experiments)
        df = postprocess_df(df, normalize)
        return df

    def create_namespace(self, name: str):
        r = self.stub.CreateNamespace(service_pb2.SimpleNamespace(namespace=name, token=self.token))
        if r.status:
            return ExperimentCollectionRemote(self.host, name, self.token)
        raise ExperimentCollectionRemoteException(r.error)

    def revoke(self, *, force=False):
        if force or input('enter YES to revoke access to all your namespaces') == 'YES':
            r = self.stub.RevokeToken(service_pb2.SimpleToken(token=self.token))
            if not r.status:
                raise ExperimentCollectionRemoteException(r.error)
        else:
            raise ExperimentCollectionRemoteException('abort')

    def grant(self, token: str):
        r = self.stub.GrantAccess(service_pb2.GrantAccessRequest(other_token=token, **self.auth))
        if not r.status:
            raise ExperimentCollectionRemoteException(r.error)



import os
import subprocess

import sys


def call(*args, shell=False):
    proc = subprocess.run(args, shell=shell, check=False)
    if proc.returncode:
        print('Error: ', *args)


setup_dir = os.path.dirname(os.path.abspath(__file__))
setup_client = os.path.join(setup_dir, 'setup_client.py')
setup_server = os.path.join(setup_dir, 'setup_server.py')

if os.path.exists(setup_client):
    call_args = sys.argv.copy()
    call_args[0] = setup_client
    call('python3', *call_args)
if os.path.exists(setup_server):
    call_args = sys.argv.copy()
    call_args[0] = setup_server
    call('python3', *call_args)



from pkg_resources import parse_requirements
from setuptools import setup


def load_requirements(filename: str) -> list:
    requirements = []
    with open(filename, 'r') as f:
        for requirement in parse_requirements(f.read()):
            extras = '[{}]'.format(','.join(requirement.extras)) if requirement.extras else ''
            requirements.append('{}{}{}'.format(requirement.name, extras, requirement.specifier))
    return requirements


def load_description(filename: str = 'README.md'):
    with open(filename, 'rt') as f:
        return f.read()


__version__ = '0.2.5'
NAME_CLIENT = 'experiment_collection'
if __name__ == '__main__':
    load_requirements('requirements_client.txt')
setup(
    name=NAME_CLIENT,
    version=__version__,
    description='Experiment collection',
    long_description=load_description(),
    long_description_content_type='text/markdown',
    url='https://github.com/AsciiShell/experiment_collection',
    author='AsciiShell (Aleksey Podchezertsev)',
    author_email='dev@asciishell.ru',
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
    ],
    keywords=['python3'],
    packages=['experiment_collection', 'experiment_collection_core'],
    package_dir={'': 'src'},
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            '{0} = {0}.__main__:main'.format(NAME_CLIENT),
        ]
    },
    include_package_data=True,
    zip_safe=False,
    install_requires=load_requirements('requirements_client.txt'),
)
